/**
 * File cache
 */

var fs = require("fs");

var cache = {};

function get(path)
{
	return cache[path];
}

function set(path, data)
{
	cache[path] = data;
}

function has(path)
{
	return (path in cache);
}

function remove(path)
{
	delete cache[path];
}

function load(path, callback, changedCallback)
{
	fs.readFile(path, "utf-8", function (err, data) {
		cache[path] = data;

		if (changedCallback) {
			try {
				fs.watch(path, { presist: false, recursive: false }, function (event, filename) {
	                if (!filename) {
	                    return;
	                }

					remove(path);
					changedCallback(event, filename);
				});
			} catch (err) {
				console.error("Error occured when watching file " + path + ": " + err.message);
				console.error(err.stack);
			}
		}

		callback(err, data);
	});
}

exports.get = get;
exports.set = set;
exports.has = has;
exports.load = load;
exports.remove = remove;
