"use strict";

let viewEngine = require("./view/view-engine");

function ClientBridgeController()
{
    '@controller { "urlPattern": "/clientBridge/" }';

	this.post = (callback, context) => {
		'@action { "allow": "post" }';
        '@returnJson';

        let bridgeName = context.params.portal;
        let funcName = context.params.name;
        let funcParams = JSON.parse(context.params.params);

        let clientBridges = viewEngine.getClientBridges();
        let bridge = clientBridges[bridgeName];

        if (!bridge) {
            return callback(new Error("Client bridge '" + bridgeName + "' not found! Please try to refresh current page."));
        }

        let func = bridge[funcName];

        if (typeof func !== "function") {
            return callback(new Error("Client bridge '" + bridgeName + "' does not have a function named '" + funcName + "' or it is not a function!"));
        }

        let actionInfo = func.getAnnotationData("action");

        let authPromise = new Promise((resolve, reject) => {
            if (actionInfo.auth) {
                if (!(actionInfo.auth instanceof Array)) {
                    actionInfo.auth = [ actionInfo.auth ];
                }

                let authPassedCount = 0;

                let authCallback = (err, auth) => {
                    if (!auth) {
                        return reject(new errors.ActionNoAuth("Client bridge " + bridgeName + "." + funcName + " not authenticated!"));
                    } else {
                        authPassedCount++;

                        if (err) {
                            return reject(err);
                        } else {
                            if (authPassedCount === actionInfo.auth.length) {
                                resolve();
                            }
                        }
                    }
                };

                for (let a of actionInfo.auth) {
                    let ss = a.split(".");
                    let modName = ss[0];
                    let funcName = ss[1];
                    let authFunc = global.__node_bubble_authenticators__[modName][funcName];

                    if (typeof authFunc !== "function") {
                        console.warn("Authenticator " + a + " is not a function or not found!");
                        continue;
                    }

                    authFunc(context).then((auth) => authCallback(null, auth)).catch((err) => authCallback(err, false));
                }
            } else {
                resolve();
            }
        });

        authPromise.then(() => {
            let returnType;

            try {
                let p = func(funcParams, context);
                if (!(p instanceof Promise)) {
                    throw new Error("Client bridge function registered as '" + name + "' does not return a promise!");
                }

                p.then((data) => callback(null, data)).catch((err) => callback(err));
            } catch (err) {
                callback(err);
            }
        }).catch((err) => callback(err));
    };
}

module.exports = ClientBridgeController;
