/**
 * Configuration module
 */

"use strict";

let fs = require("fs");
let path = require("path");

/**
 * Return a default configuration when config.json is not available
 *
 * @returns Default configuration object
 */
function defaultConfig()
{
	return {
		host: "localhost",
		port: 8000,
		prefix: "/",
		webRoot: ".",					// Deprecated for WebRoots
		defaultUrl: "/index.html",
		maxPostSize: 2097152,
		precompileViews: true,			// Deprecated for XML views
		logDir: "./logs",
		accessible: [],					// Deprecated for WebRoots
		fileUrlMap: {},
		insecure: false,
		enableWebRootChar: true,
		webRootChar: "~"
	};
}

let rootDir = path.dirname(require.main.filename);
let configData;

try {
	let configPath = path.join(rootDir, "bubble-config.json");

	console.info("Loading configuration from " + configPath);

	configData = JSON.parse(fs.readFileSync(configPath));

	let defConfig = defaultConfig();
	for (let key of Object.keys(defConfig)) {
		if (!configData[key]) {
			configData[key] = defConfig[key];
		}
	}

	console.info("Configuration loaded.");
} catch (e) {
	console.warn(e);
	console.warn("Default configuration loaded.");

	configData = defaultConfig();
}

module.exports = configData;
