/**
 * DEPRECATED - Process controller actions
 */

require("../utils/annotation");

var errors = require("../errors");
var templateCache = require("../caches/templates");
var fileCache = require("../caches/files");
var renderer = require("../view/renderer");
var compositor = require("../view/compositor");
var stream = require("stream");
var requestUtils = require("../utils/request");
var configs = require("../conf/configs");
var Cookies = require("cookies");
var path = require("path");
var streamUtils = require("../utils/stream");

function getActionNameFromUrl(url)
{
	if (!url) {
		return null;
	}

	var start = url.lastIndexOf("/");
	if (start < 0) {
		return url;
	}

	return url.substring(start + 1);
}

function getFunction(name, controller)
{
	if (controller[name]) {
		return controller[name];
	}

	return global[name];
}

function processView(viewInfo, controllerPath, model, context, compileOnly, callback)
{
	var realViewPath = path.join(path.dirname(controllerPath), viewInfo.path);

	var afterFileLoad = function (data) {
		var templatePreprocessed = compositor.parse(data, path.dirname(realViewPath));
		var template = renderer.render(templatePreprocessed, model, context, realViewPath);
		templateCache.set(realViewPath, template.cache);

        if (!compileOnly) {
            console.info("Compiled view page " + viewInfo.path + " for " + controllerPath);
    		callback(null, template.cache(model, context, configs.prefix));
        } else {
            callback(null, null);
        }
	};

	if (!templateCache.has(realViewPath)) {
		if (!fileCache.has(realViewPath)) {
			fileCache.load(realViewPath, function (err, data) {
                if (err) {
                    return callback(err);
                }

				afterFileLoad(data);
			}, function (event, filename) {
				console.info(filename + " has " + event);
				templateCache.remove(realViewPath);
			});
		} else {
			afterFileLoad(fileCache.get(realViewPath));
		}
	} else {
		var template = templateCache.get(realViewPath);
		var finalData;

		try {
			finalData = template(model, context, configs.prefix);
		} catch (err) {
			console.error("Error occured when processing view " + realViewPath + ": " + err.message);
			console.error(err.stack);
			return callback(err);
		}

		callback(null, finalData);
	}
}

function getAction(urlData, controller)
{
	var actionName = getActionNameFromUrl(urlData.pathname);
	var action = controller[actionName];

	if ((!action) || (!action.hasAnnotation("action"))) {
		return null;
	} else {
		return { actionName: actionName, action: action };
	}
}

function callAction(controllerInfo, urlData, action, data, files, cookies, request, response,
		afterAction, callback)
{
	var context = {
		params: data,
		files: files,
		cookies: cookies,
		request: request,
		response: response,
		configs: configs
	};

	var controller = controllerInfo.controller;

	action(function (err, result, mimeType, size) {
		if (err) {
			callback(err);
			return;
		}

		var processAction = function () {
			if (action.hasAnnotation("view")) {
				var viewInfo = action.getAnnotationData("view");
				processView(viewInfo, controllerInfo.path, result, context, false, function (err, data) {
					if (err) {
						callback(err);
						return;
					}

					var dataStream = streamUtils.dataToStream(data);
					callback(null, dataStream, "text/html", Buffer.byteLength(data, "utf8"));
				});

				return;
			}

			var data = null;
			mimeType = mimeType ? mimeType : "application/octet-stream";
			var length = size ? size : 0;

			if (action.hasAnnotation("returnJson")) {
				mimeType = "application/json";
				data = JSON.stringify(result);

				if ((data === null) || (data === undefined)) {
					data = "{}";
				}

				length = Buffer.byteLength(data, "utf8");
			} else if (action.hasAnnotation("return")) {
				var returnInfo = action.getAnnotationData("return");
				mimeType = returnInfo.type;
			}

			if ((data === null) || (data === undefined)) {
				if ((result !== null) && (result !== undefined)) {
					data = result;
				} else {
					data = "";
					length = 0;
				}
			}

			if ((data) && (length <= 0)) {
				length = data.length;
			}

			callback(null, streamUtils.dataToStream(data), mimeType, length);
		};

		if (afterAction) {
			if (!(afterAction instanceof Array)) {
				afterAction = [ afterAction ];
			}

			var afterActionCalled = function (err, data) {
				afterActionCounter++;

				if (err) {
					return callback(err);
				}

				result = data;

				if (afterActionCounter >= afterAction.length) {
					processAction();
				}
			};

			var afterActionCounter = 0;

			for (var i = 0; i < afterAction.length; i++) {
				var func = getFunction(afterAction[i], controller);
				if (!func) {
					return callback(new errors.NoSuchAction("After action " + afterAction[i] + " not found!"));
				}

				func(afterActionCalled, context, result);
			}
		} else {
			processAction();
		}
	}, context);
}

/**
 * Find and execute controller actions by URL
 *
 * @param {object} request - The request
 * @param {object} response - The response
 * @param {object} urlData - URL data
 * @param {object} controllerInfo - Controller information structure containing a controller holding actions and
 * 									controller file path
 * @param {function} callback - Callback function with signature fn(err: Error, stream: ReadStream,
 *								mimeType: string, length: integer)
 */
function process(request, response, urlData, controllerInfo, callback)
{
	var controller = controllerInfo.controller;

	var a = getAction(urlData, controller);
	if (!a) {
		return callback(new errors.NoSuchAction("Controller " + controller + " doesn't have action " + urlData.href));
	}

	var action = a.action;
	var actionName = a.actionName;
	var cookies = new Cookies(request, response);
	var actionInfo = action.getAnnotationData("action");
	var allowMethod = true;
	var skipRequestParse = false;
	var authPassed = false;
	var afterAction = null;

	var afterAuth = function () {
		if (!authPassed) {
			return;
		}

		if (!skipRequestParse) {
			requestUtils.getRequestData(request, function (err, data, files) {
				if (err) {
					callback(err);
					return;
				}

				if (!actionInfo.insecure) {
					if ((!requestUtils.checkRequestParameters(data)) || (!requestUtils.checkRequestParameters(request.headers.cookies))) {
						return callback(new errors.InsecureRequest("Insecure data detected in request!"));
					}
				}

				callAction(controllerInfo, urlData, action, data, files, cookies, request, response,
						afterAction, callback);
			});
		} else {
			callAction(controllerInfo, urlData, action, null, null, cookies, request, response,
					afterAction, callback);
		}
	};

	if (actionInfo) {
		if (actionInfo.allow !== undefined) {
			allowMethod = false;
			actionInfo.allow = actionInfo.allow.toLowerCase();

			if (actionInfo.allow instanceof Array) {
				for (var i = 0; i < actionInfo.allow.length; i++) {
					if (actionInfo.allow[i] === request.method.toLowerCase()) {
						allowMethod = true;
						break;
					}
				}
			} else {
				if (actionInfo.allow === request.method.toLowerCase()) {
					allowMethod = true;
				}
			}
		}

		if (actionInfo.skipRequestParse === true) {
			skipRequestParse = true;
		}

		if (actionInfo.after) {
			afterAction = actionInfo.after;
		}

		// NOTE: This section must be last
		if (actionInfo.auth) {
			var authContext = {
				actionName: actionName,
				params: urlData.query,
				cookies: cookies,
				request: request,
				response: response
			};

			var authFuncNames;

			if (actionInfo.auth instanceof Array) {
				authFuncNames = actionInfo.auth;
			} else {
				authFuncNames = [ actionInfo.auth ];
			}

			var authPassedCount = 0;
			var authCallback = function (err, auth) {
				if (!auth) {
					return callback(new errors.ActionNoAuth("Action " + actionName + " not authenticated!"));
				} else {
					authPassed = true;
					authPassedCount++;

					if (err) {
						return callback(err);
					} else {
						if (authPassedCount === authFuncNames.length) {
							afterAuth();
						}
					}
				}
			};

			for (var j = 0; j < authFuncNames.length; j++) {
				var authFunc = getFunction(authFuncNames[j], controller);
				authFunc(authCallback, authContext);
			}
		} else {
			authPassed = true;
		}
	} else {
		authPassed = true;
	}

	if (!allowMethod) {
		callback(new errors.MethodNotAllowed("Action " + actionName + " doesn't like " + request.method));
		return;
	}

	afterAuth();
}

function canProcess(urlData, controller)
{
	return (getAction(urlData, controller) !== null);
}

exports.process = process;
exports.canProcess = canProcess;
exports.processView = processView;
