"use strict";

let mime = require("mime");
let configs = require("./conf/configs");
let fs = require("fs");
let path = require("path");
let errors = require("./errors");

function ExternalFileResolverController()
{
	'@controller { "urlPattern": "/externalFileResolver" }';

	this.externalFileResolver = (callback, context) => {
		'@action';

		let realPath = path.resolve(path.join(configs.webRoot, context.params.path));
		let allow = false;

		for (let i = 0; i < configs.accessible.length; i++) {
			let allowedPath = path.resolve(path.join(configs.webRoot, configs.accessible[i]));
			if ((realPath.indexOf(allowedPath) === 0) || (realPath.indexOf(configs.accessible[i]) === 0)) {
				allow = true;
			}
		}

		if (!allow) {
			console.warn("Path " + realPath + " is not allowed to access!");
			return callback(new errors.FileNotFound("Requested file " + context.params.path + " not found!"));
		}

		fs.stat(realPath, (err, stats) => {
			if (err) {
				return callback(err);
			}

			if (context.request.headers["if-modified-since"]) {
				let modifyCheckDate = Date.parse(context.request.headers["if-modified-since"]);

				if (modifyCheckDate >= stats.mtime) {
					context.response.writeHead(304);
					context.response.end();
					callback(null, null, null, 0, true);
					return;
				}
			}

			context.response.setHeader("Last-Modified", stats.mtime.toUTCString());

			callback(null, fs.createReadStream(realPath), mime.lookup(realPath), stats.size);
		});
	};
}

module.exports = ExternalFileResolverController;
