/**
 * node-bubble: HTTP server framework for node.js
 */

"use strict";

let httpServer = require("./server/http-server");
let configs = require("./conf/configs");
let pkgInfo = require("./package.json");

exports.initLogger = () =>
{
	require("./utils/logger");
};

exports.start = () =>
{
	return new Promise((resolve, reject) => {
		exports.initLogger();

		if (!global.__node_bubble_started__) {
			global.__node_bubble_started__ = true;
		} else {
			return resolve(configs);
		}

		console.info("node-bubble " + pkgInfo.version + " is starting...");

		httpServer.start().then(() => {
			console.info("node-bubble started.");
			resolve(configs);
		}).catch((err) => {
			console.error("Error occured when starting node-bubble: " + err.message);
			console.error(err.stack);
			reject(err);
		});
	});
};

exports.addFileScanCallback = httpServer.addFileScanCallback;

exports.Service = require("./services/service");
exports.WebSocketService = require("./services/websocket-service");
