"use strict";

let Service = require("./service");
var Cookies = require("cookies");
var requestUtils = require("../utils/request");

if (!global.__node_bubble_service_list__) {
    global.__node_bubble_service_list__ = new Map();
}

function putService(urlPattern, instance)
{
    global.__node_bubble_service_list__.set(urlPattern, instance);
}

exports.getLoadedServices = () => global.__node_bubble_service_list__;

exports.loadService = (path) =>
{
    let s = require(path);
    if (!s) {
        console.error("Service module " + path + " does not export a valid service!");
        return;
    }

    try {
        let instance = new s();

        if ((!instance._service) || (!(instance instanceof Service))) {
            console.error("Service module " + path + " is not a valid service!");
            return;
        }

        if (typeof instance.init !== "function") {
            console.error("Service module " + path + " does not have an init method!");
            return;
        }

        if (!instance._service.route) {
            console.warn("Service module " + path + " has no route defined!");
        }

        instance.init();

        putService(instance._service.route || Math.random().toString(), instance);

        console.info("Loaded service at " + path);
    } catch (err) {
        console.error("Error occured when initializing service " + path + ": " + err.message);
        console.error(err.stack);
        return;
    }
};

exports.doService = (instance, request, response, urlData, callback) =>
{
    if (instance._service.handoff) {
        return callback(null, null, null, 0);
    }

    instance.dispatch(request, response, urlData)
        .then((data) => callback(null, data.stream, data.mimeType, data.length))
        .catch((err) => callback(err));
};
