/**
 * Annotation support library
 *
 * Usage:
 *
 * function foo ()
 * {
 *     '@anno { "a": 1, "b": 2 }';
 *
 *     doSomething();
 * }
 *
 * foo.hasAnnotation("anno") = true
 * foo.getAnnotationData("anno") = { a: 1, b: 2 }
 */

"use strict";

/**
 * Check if the function has specified annotation
 *
 * @param {string} annotation - Annotation to check
 *
 * @returns If the annotation exists
 */
Function.prototype.hasAnnotation = function (annotation) {
	let func = this.toString();
	return (func.indexOf("@" + annotation) >= 0);
};

/**
 * Get annotation data of function
 *
 * @param {string} annotation - Annotation to check
 *
 * @returns Data of this annotation
 */
Function.prototype.getAnnotationData = function (annotation) {
	let func = this.toString();
	let dataStart = func.indexOf("@" + annotation) + annotation.length + 1;
	let dataEnd = func.indexOf("';", dataStart);
	if (dataStart >= dataEnd) {
		return {};
	}

	let dataString = func.substring(dataStart, dataEnd);

	try {
		return JSON.parse(dataString);
	} catch (e) {
		return {};
	}
};
