"use strict";

if (global.testMode) {
	let dummyLogger = {
		info: function () {},
		warn: function () {},
		error: function () {}
	};

	exports.accessLogger = dummyLogger;
} else {
	let log4js = require("log4js");
	let configs = require("../conf/configs");
	let path = require("path");
	let mkdirp = require("mkdirp");
	let fs = require("fs");

	fs.stat(configs.logDir, (err, stats) => {
		let doConfLog = () => {
			console.info("Log directory: " + configs.logDir);

			log4js.configure({
				appenders: [
				    {
				    	type: "console"
				    },
				    {
				    	type: "dateFile",
				    	filename: path.join(configs.logDir, "access.log"),
				    	pattern: "-yyyy-MM-dd",
				    	alwaysIncludePattern: false,
				    	category: "access"
				    },
				    {
				    	type: "dateFile",
				    	filename: path.join(configs.logDir, "output.log"),
				    	pattern: "-yyyy-MM-dd",
				    	alwaysIncludePattern: false
				    }
				],
				replaceConsole: true
			});
		};

		let doCreatePath = () => {
			mkdirp(configs.logDir, (err) => {
				if (err) {
					console.error(err);
				}

				doConfLog();
			});
		};

		if (err) {
			return doCreatePath();
		}

		if (!stats.isDirectory) {
			fs.rmdir(configs.logDir, (err) => {
				if (err) {
					console.error(err);
				}

				doCreatePath();
			});
		} else {
			doConfLog();
		}
	});

	let accessLogger = log4js.getLogger("access");
	accessLogger.setLevel("ALL");

	exports.accessLogger = accessLogger;
}
