/**
 * HTML template block compositor
 *
 * NOTE: Deprecated in 0.2, please use view-engine.js instead!
 */

var configs = require("../conf/configs");
var fs = require("fs");
var path = require("path");

function processAttributeLine(str)
{
	var strArray = str.match(/[^\s=['"]*=['"][^'"]*['"]/g);
	var attrs = {};

	if (!strArray) {
		return {};
	}

	for (var i = 0; i < strArray.length; i++) {
		var arr = strArray[i].split(/=['"]/);
		attrs[arr[0]] = arr[1].substring(0, arr[1].length - 1);
	}

	return attrs;
}

function regexIndexOf(str, regex, startpos)
{
    var indexOf = str.substring(startpos || 0).search(regex);
    return (indexOf >= 0) ? (indexOf + (startpos || 0)) : indexOf;
}

/**
 * Parse template block data and composite blocks
 *
 * @param {string} data - Template data
 * @param {string} viewDir - Template path without filename
 *
 * @returns Composited template data
 */
function parse(data, viewDir)
{
	var blockTagStart = 0;
	while (blockTagStart >= 0) {
		blockTagStart = data.indexOf("<block ", blockTagStart);
		if (blockTagStart < 0) {
			break;
		}

		var blockTagEnd = regexIndexOf(data, /['"]\s*>/, blockTagStart);
		if (blockTagEnd < 0) {
			console.error("Template block tag is not closed!");
			blockTagStart++;
			continue;
		} else {
			blockTagEnd++;
		}

		var blockTag = data.substring(blockTagStart + 7, blockTagEnd);
		var blockAttributes = processAttributeLine(blockTag);

		if (!blockAttributes.file) {
			console.error("Template block tag has no file attribute!");
			blockTagStart++;
			continue;
		}

		var blockEnd = data.indexOf("</block>", blockTagEnd);
		if (blockEnd < 0) {
			console.error("Unclosed block tag at " + blockTagEnd);
			console.error("Current data:");
			console.error(data);

			blockTagStart++;
			continue;
		}

		var blockTemp = data.substring(blockTagEnd, blockEnd);
		var blockTagCount = (blockTemp.match(/<block/g) || []).length;
		var lastBlockEnd = blockEnd;
		if (blockTagCount > 0) {
			while (blockTagCount > 0) {
				blockEnd = data.indexOf("</block>", blockEnd + 1);
				blockTagCount--;

				if (data.substring(lastBlockEnd, blockEnd).indexOf("<block ") >= 0) {
					blockTagCount++;
				}

				lastBlockEnd = blockEnd;
			}

			if (blockEnd < 0) {
				console.error("Unbalanced block tags!");
				blockTagStart++;
				continue;
			}
		}

		var blockPath = path.resolve(path.join(viewDir, blockAttributes.file));
		var deltaPath = path.relative(viewDir, path.dirname(blockPath));
		var templateData;

		try {
			templateData = fs.readFileSync(blockPath, "utf-8");
		} catch (err) {
			console.error("Error occured when compositing view: " + err.message);
			console.error(err.stack);
			return "";
		}

		if (templateData.indexOf("<!--[# body ]-->") >= 0) {
			var blockBody = data.substring(blockTagEnd + 1, blockEnd);
			templateData = templateData.replace(/<\!--\[# body \]-->/g, blockBody);
		}

		if (templateData.indexOf("$DELTA_PATH") >= 0) {
			templateData = templateData.replace(/\$DELTA_PATH/g, deltaPath);
		}

		for (var key in blockAttributes) {
			if (!blockAttributes.hasOwnProperty(key)) {
				continue;
			}

			var attrHolder = "<\\!--\\[# " + key + " \\]-->";
			templateData = templateData.replace(new RegExp(attrHolder, "g"), blockAttributes[key]);

			var attrCodeHolder = "([^a-zA-Z_])attr." + key + "([^a-zA-Z_])";
			templateData = templateData.replace(new RegExp(attrCodeHolder, "g"),
					"$1" + blockAttributes[key].replace(/<\!--\[=(.*?)\]-->/g, "($1)") + "$2");
		}

		var templateParsedData = parse(templateData, viewDir);

		data = data.substring(0, blockTagStart) + templateParsedData + data.substring(blockEnd + 8);

		blockTagStart++;
	}

	return data;
}

exports.parse = parse;
