"use strict";

let configs = require("../conf/configs");
let RenderUtils = require("./render-utils");
let TagUtils = require("./tag-utils");
let StringUtils = require("./string-utils");

function _clientBridgesToClientJs(clientBridges)
{
    let str = "<script type=\"text/javascript\">\\n";
    let added = false;

    for (let key of Object.keys(clientBridges)) {
        let bridge = clientBridges[key];

        str += "var " + key + " = {\\n";

        for (let func of Object.keys(bridge)) {
            str += "\"" + func + "\"\: function (params, callback) {\\n";
            str += "$.post(\"" + configs.prefix + "clientBridge/post\", { portal: \"" + key + "\", name: \"" + func + "\", params: JSON.stringify(params) }).done((data, textStatus, jqXHR) => {\\n" +
                "callback(null, data);\\n" +
                "}).fail((jqXHR, textStatus, errorThrown) => {\\n" +
                "callback(new Error(errorThrown));\\n" +
                "});\\n},\\n";
        }

        str += "};\\n";
        added = true;
    }

    str += "</script>\\n";

    if (!added) {
        return "";
    }

    return str.replace(/'/g, "\\'");
}

exports.render = (data, clientBridges) =>
{
    let template = data.replace(/\r\n/g, "\n").replace(/\n/g, "\\n");
    let buf = "var buf = '';\n";

    let codePhase = template.split("<!--[").join("';\n").split("]-->").join("\nbuf += '");
    let processed = codePhase.split("${").join("' + (").split("}$").join(") + '");
    buf += "buf += '" + processed + "';\n";

    let pos = buf.indexOf("</body>");
    let cbCode = _clientBridgesToClientJs(clientBridges);

    if (pos < 0) {
        buf += "buf += '" + cbCode + "';\n";
    } else {
        buf = buf.substring(0, pos) + "\\n" + cbCode + "\\n" + buf.substring(pos, buf.length);
    }

    buf += "return buf;";

    let asyncRenderFunction;
    let GeneratorFunction = Object.getPrototypeOf(function*(){}).constructor;

    try {
        /* jslint evil: true */
        asyncRenderFunction = new GeneratorFunction("model", "params", "context", "root", "deferredList", "R", "T", "S", buf);
    } catch (err) {
        console.error("Error occured when compiling template: " + err.message);
        console.error(err.stack);
        console.error("Current buffer: ");
        console.error(buf);
        console.error("Before compile: ");
        console.error(template);
    }

    let renderUtils = new RenderUtils("/");
    let tagUtils = new TagUtils();
    let stringUtils = new StringUtils();

    return {
        cache: (model, context, deferredList) => asyncRenderFunction(model, context.params, context, configs.prefix, deferredList, renderUtils, tagUtils, stringUtils)
    };
}
