"use strict";

let ViewTag = require("../view-tag");
let ViewTagUtils = require("../view-tag-utils");
let Util = require("util");
let path = require("path");
let errors = require("../../errors");

class AuthTag extends ViewTag
{
    render(attrs, content) {
        return ViewTagUtils.putDeferred((context, moduleName, funcName) => {
            return new Promise((resolve, reject) => {
                let m = global.__node_bubble_authenticators__[moduleName];
                if (typeof m[funcName] !== "function") {
                    return reject(new Error("Authenticator action " + funcName + " in module " + moduleName + " is not a function!"));
                }

                m[funcName](context).then((auth) => {
                    if (!auth) {
                        return reject(new errors.ActionNoAuth("View " + context.request.url + " is not authenticated!"));
                    }

                    resolve("");
                }).catch((err) => reject(err));
            });
        }, attrs.module, attrs.action);
    }
}

module.exports = AuthTag;
