"use strict";

let configs = require("../conf/configs");

function DeferredValue(fn, params)
{
	this.functor = fn;
	this.params = params;
}

exports.DeferredValue = DeferredValue;

function DeferredVariable(varName, fn, params)
{
	this.varName = varName;
	this.functor = fn;
	this.params = params;
}

exports.DeferredVariable = DeferredVariable;

exports.putDeferred = (fn, ...params) =>
{
	return new DeferredValue(fn, params);
};

exports.putDeferredVariable = (varName, fn, ...params) =>
{
	return new DeferredVariable(varName, fn, params);
};

exports.resolveUrl = (url) =>
{
	if ((url.indexOf("://") >= 0) || (url.startsWith("//"))) {
		return url;
	}

	if (url.startsWith("/")) {
		return configs.prefix.endsWith("/") ? configs.prefix.substring(0, configs.prefix.length - 1) + url : configs.prefix + url;
	}

	return url;
};

exports.putClientScript = (fn, ...params) =>
{
	let paramString = "";

	for (let p of params) {
		if (typeof p === "string") {
			paramString += "\"" + p.replace("'", "\\'") + "\",";
		} else if (typeof p === "number") {
			paramString += p + ",";
		} else if (typeof p === "function") {
			paramString += p.toString() + ",";
		} else if ((p === undefined) || (p === null)) {
			paramString += "null,";
		} else {
			paramString += JSON.stringify(p) + ",";
		}
	}

	paramString = paramString.substring(0, paramString.length - 1);

	let str = "<script type=\"text/javascript\">\n";
	str += "(" + fn.toString() + ")(" + paramString + ");\n";
	str += "</script>\n";

	return str;
};
